<?php

require_once(dirname(__FILE__).'/cache.php');
require_once(dirname(__FILE__).'/config_ui.php');
require_once(dirname(__FILE__).'/const.php');
require_once(dirname(__FILE__).'/option.php');
require_once(dirname(__FILE__).'/postmeta.php');
require_once(dirname(__FILE__).'/runtime.php');
require_once(dirname(__FILE__).'/write_ui.php');

// ----------------------------------------------------------------------------
// the DiscloseSecret_Manager class handles the plugin initialization phase,
// assuring the infrastructure is set up properly
// ----------------------------------------------------------------------------

class DiscloseSecret_Manager
{
	var $m_activation_status = DiscloseSecret_STATUS_UNINITIALIZED;
	var $m_cache = NULL;
	var $m_runtime = NULL;
	var $m_config_ui = NULL;
	var $m_write_ui = NULL;

	// ----------------------------------------------------------------------------
	// init
	// ----------------------------------------------------------------------------

	function DiscloseSecret_Manager()
	{
		// Just in case someone's loaded up the page standalone for whatever reason,
		// make sure it doesn't crash in an ugly way
		if (function_exists('load_plugin_textdomain'))
			load_plugin_textdomain(DiscloseSecret_PLUGIN_ID, DiscloseSecret_DIR. '/languages');

		global $wp_version;
		if (!isset($wp_version))
			die(__('This page must be loaded as part of WordPress', DiscloseSecret_PLUGIN_ID));

		add_filter('init', array(&$this, 'filter_init'));
	}

	function filter_init()
	{
		$this->m_cache =& new DiscloseSecret_Cache();
		$option =& $this->m_cache->get_option();
		$this->m_activation_status = $option->get_cooperation_status();

		// don't bother with anything more unless someone is logged in
		if (is_user_logged_in())
		{
			// Does not work with WP 2.0.4 on Windows; see trac ticket #3002; this is not vital, so just don't care
			register_activation_hook('shutdown', array(&$this, 'filter_activation'));

			// make sure private pages aren't cached publicly
			header('Cache-Control: private');
			header('Pragma: no-cache');

			$this->m_runtime =& new DiscloseSecret_Runtime($this->m_cache, $this->m_activation_status);
			$this->m_config_ui =& new DiscloseSecret_ConfigUi($this->m_cache, $this->m_activation_status);
			$this->m_write_ui =& new DiscloseSecret_WriteUi($this->m_cache, $this->m_activation_status);
		}
	}

	function filter_activation()
	{
		if ($this->m_activation_status == DiscloseSecret_STATUS_UNINITIALIZED
			|| $this->m_activation_status == DiscloseSecret_STATUS_PLUGIN_VERSION_MISMATCH)
			return;

		// turn off the overrider for next plugin activation
		$option =& $this->m_cache->get_option();
		$option->set_override_plugin_deactivation(false);
		$option->save();
	}

	function &get_cache()
	{
		return $this->m_cache;
	}

	function is_active()
	{
		return $this->m_activation_status == DiscloseSecret_STATUS_OKAY;
	}
}

?>