<?php

require_once(dirname(__FILE__).'/const.php');

define('DiscloseSecret_ACTION_MIGRATE_POSTLEVELS', 'migrate_post_levels');
define('DiscloseSecret_CONFIG_FORM_ID_POSTLEVELS', 'post_levels_migrate');

define('DiscloseSecret_POSTLEVELS_META', 'post_level');
define('DiscloseSecret_POSTLEVELS_META_POST_KEY', 'postlevels_post_key');
define('DiscloseSecret_POSTLEVELS_META_USER_KEY', 'postlevels_user_key');
define('DiscloseSecret_POSTLEVELS_ROLE_ADMIN', 'post_level_admin');

// ----------------------------------------------------------------------------
// the DiscloseSecret_MigratorPostLevels migrates the Post Levels plugin
// to Disclose-Secret
// ----------------------------------------------------------------------------

class DiscloseSecret_MigratorPostLevels
{
	var $m_status;

	function DiscloseSecret_MigratorPostLevels()
	{
		$this->m_status = DiscloseSecret_MIGRATION_NOT_NEEDED;
	}

	function get_name()
	{
		return 'MigratorPostLevels';
	}

	function migrate($interactive = true)
	{
		$this->m_status = $this->migrate_plugin($interactive);
		return $this->m_status;
	}

	function migrate_plugin($interactive = true)
	{
		global $wpdb, $wp_roles;

		if ($interactive)
			$do_migration = isset($_POST[DiscloseSecret_ACTION_MIGRATE_POSTLEVELS]);
		else
			$do_migration = true;

		// get the Post Level options
		$postlevels_post_meta_keys[] = DiscloseSecret_POSTLEVELS_META;
		$postlevels_post_meta_key = get_option(DiscloseSecret_POSTLEVELS_META_POST_KEY);
		if ($postlevels_post_meta_key !== false)
			$postlevels_post_meta_keys[] = $postlevels_post_meta_key;
		$postlevels_post_meta_in = "'". implode("', '", $postlevels_post_meta_keys). "'";

		$postlevels_user_meta_keys[] = '';
		$postlevels_user_meta_key = get_option(DiscloseSecret_POSTLEVELS_META_USER_KEY);
		if ($postlevels_user_meta_key !== false)
			$postlevels_user_meta_keys[] = $postlevels_user_meta_key;
		$postlevels_user_meta_in = "'". implode("', '", $postlevels_user_meta_keys). "'";

		// find out whether plugin is installed
		$postlevels_installed = false;
		$query = "
			SELECT COUNT(*)
			FROM {$wpdb->postmeta}
			WHERE meta_key IN ($postlevels_post_meta_in)
			AND meta_value IN (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
		";
		if ($wpdb->get_var($query) > 0 && !in_array(DiscloseSecret_META_CAP, $postlevels_post_meta_keys))
			$postlevels_installed = true;

		// find out whether user level migration has been done
		$postlevels_user_migration = false;
		$query = "
			SELECT COUNT(*)
			FROM {$wpdb->usermeta}
			WHERE meta_key IN ($postlevels_user_meta_in)
			AND meta_value IN (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
		";
		if ($wpdb->get_var($query) > 0)
			$postlevels_user_migration = true;

		if (!$postlevels_installed)
			return DiscloseSecret_MIGRATION_NOT_NEEDED;
		if (!$do_migration)
			return DiscloseSecret_MIGRATION_SKIPPED;
		if ($interactive)
			check_admin_referer(DiscloseSecret_ACTION_MIGRATE_POSTLEVELS);

		// do migration if requested
		$postlevels_cap_prefix = DiscloseSecret_CAPABILITY_PREFIX_LEVEL;

		// create new capabilities if post levels 1.0 has migrated user levels
		if ($postlevels_user_migration)
		{
			// select user levels to convert to capablities
			$postlevels_cap_prefix = DiscloseSecret_CAPABILITY_PREFIX_READ. $postlevels_cap_prefix;
			$query = "
				SELECT user_id, meta_value
				FROM {$wpdb->usermeta}
				WHERE meta_key IN ($postlevels_user_meta_in)
				AND meta_value IN (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
			";
			$wpdb->query($query);
			$s = $wpdb->get_results($query);
			if (!is_array($s))
				$s = array();

			// create capabilities for this user and administrator role
			foreach ($s as $i)
			{
				// the admin role is needed because some role manager otherwise
				// do not recognize capabilities only assigned to users
				add_role(DiscloseSecret_POSTLEVELS_ROLE_ADMIN,
					__("Post Levels Administrator", DiscloseSecret_PLUGIN_ID));
				$role = get_role(DiscloseSecret_POSTLEVELS_ROLE_ADMIN);
				$role->add_cap($postlevels_cap_prefix. $i->meta_value);
				$user =& new WP_User($i->user_id);
				$user->add_cap($postlevels_cap_prefix. $i->meta_value);
			}

			// select post levels to convert to capablities
			$query = "
				SELECT DISTINCT meta_value
				FROM {$wpdb->postmeta}
				WHERE meta_key IN ($postlevels_post_meta_in)
				AND meta_value IN (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
			";
			$wpdb->query($query);
			$s = $wpdb->get_results($query);
			if (!is_array($s))
				$s = array();

			// create post capabilities for administrator role
			foreach ($s as $i)
			{
				add_role(DiscloseSecret_POSTLEVELS_ROLE_ADMIN,
					__("Post Levels Administrator", DiscloseSecret_PLUGIN_ID));
				$role = get_role(DiscloseSecret_POSTLEVELS_ROLE_ADMIN);
				$role->add_cap($postlevels_cap_prefix. $i->meta_value);
			}

			// delete obsolete post levels user metas
			$query = "
				DELETE
				FROM {$wpdb->usermeta}
				WHERE meta_key IN ($postlevels_user_meta_in)
			";
			$wpdb->query($query);

			$postlevels_user_migration = false;
		}

		// changing post meta key values to that of Disclose-Secret
		$query = "
			UPDATE {$wpdb->postmeta}
			SET meta_key = '". DiscloseSecret_META_CAP. "',
				meta_value = CONCAT('$postlevels_cap_prefix', meta_value)
			WHERE meta_key IN ($postlevels_post_meta_in)
			AND meta_value IN (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
		";
		$wpdb->query($query);

		return DiscloseSecret_MIGRATION_DONE;
	}

	function print_form()
	{
		if ($this->m_status == DiscloseSecret_MIGRATION_SKIPPED)
		{
?>
	<div class="wrap">
		<h2><?php _e('Migration from Post Levels', DiscloseSecret_PLUGIN_ID); ?></h2>
		<p><?php _e('Disclose-Secret has found that your blog used a version of the <a href="http://fortes.com/projects/wordpress/postlevels">Post Levels</a> plugin. If both plugins are active the functionality of each may behave unexpected! You can easily migrate your post levels to Disclose-Secret by pressing the button below.', DiscloseSecret_PLUGIN_ID); ?></p>
		<p><?php _e('The migration will either convert the previous set post_level to equivalent &quot;level_X&quot; capabilities or in case you have used Post Levels &quot;Migrate user levels&quot; feature it will convert the used post levels to a set of newly created &quot;read_level_X&quot; capabilites (where X is the number of the legacy user level). In case of newly created capabilities these will be assigned in equivalent manner to the users as they were previously assigend by the Post Levels plugin. Additionally a new &quot;Post Levels Administrator&quot; role will be created were all of the newly created capabilities been assigned to. These admin role is not yet assigned to any user after migration. You are free to delete it if you do not want to use it. It is just meant to simplify role/capability management after migration.', DiscloseSecret_PLUGIN_ID); ?></p>
		<p><?php _e('Summary: After migration your blog behaves the same way as with Post Levels but is configured differently. You also should consider to install a Role Manager. See the common plugin repositories for implementations.', DiscloseSecret_PLUGIN_ID); ?></p>
		<p><?php _e('Migration was tested with Post Levels version 1.0 but should work with all previous versions.', DiscloseSecret_PLUGIN_ID); ?></p>
		<p><strong><?php _e('Note, that migration can not be undone by this plugin once you have pressed the button below.', DiscloseSecret_PLUGIN_ID); ?></strong></p>
		<form action="" method="post" id="<?php echo DiscloseSecret_CONFIG_FORM_ID_POSTLEVELS; ?>">
			<p class="submit">
				<input type="submit" name="<?php echo DiscloseSecret_ACTION_MIGRATE_POSTLEVELS; ?>" value="<?php _e('Migrate Post Levels &raquo;', DiscloseSecret_PLUGIN_ID); ?>" />
				<?php wp_nonce_field(DiscloseSecret_ACTION_MIGRATE_POSTLEVELS); ?>

			</p>
		</form>
	</div>
<?php
		}
	}
}

?>