<?php

require_once(dirname(__FILE__).'/const.php');

define('DiscloseSecret_ACTION_MIGRATE_PRIVATECATEGORIES', 'migrate_private_categories');
define('DiscloseSecret_CONFIG_FORM_ID_PRIVATECATEGORIES', 'private_categories_migrate');

define('DiscloseSecret_PRIVATECATEGORIES_OPTION_CATS', 'wwpc_cats');
define('DiscloseSecret_PRIVATECATEGORIES_OPTION_LEVEL', 'wwpc_level');

// ----------------------------------------------------------------------------
// the DiscloseSecret_MigratorPrivateCategories class migrates the
// Private Categories plugin to Disclose-Secret
// ----------------------------------------------------------------------------

class DiscloseSecret_MigratorPrivateCategories
{
	var $m_status;

	function DiscloseSecret_MigratorPrivateCategories()
	{
		$this->m_status = DiscloseSecret_MIGRATION_NOT_NEEDED;
	}

	function get_name()
	{
		return 'MigratorPrivateCategories';
	}

	function migrate($interactive = true)
	{
		$this->m_status = $this->migrate_plugin($interactive);
		return $this->m_status;
	}

	function migrate_plugin($interactive = true)
	{
		global $wpdb;

		if ($interactive)
			$do_migration = isset($_POST[DiscloseSecret_ACTION_MIGRATE_PRIVATECATEGORIES]);
		else
			$do_migration = true;

		// get the plugin options
		$privatecategories_level = get_option(DiscloseSecret_PRIVATECATEGORIES_OPTION_LEVEL);
		if ($privatecategories_level < 0)
			$privatecategories_level = 0;
		else if ($privatecategories_level > 10)
			$privatecategories_level = 10;
		$privatecategories_cats = get_option(DiscloseSecret_PRIVATECATEGORIES_OPTION_CATS);
		$privatecategories_installed = false;
		if (is_array($privatecategories_cats) && count($privatecategories_cats) > 0)
			$privatecategories_installed = true;

		if (!$privatecategories_installed)
			return DiscloseSecret_MIGRATION_NOT_NEEDED;
		if (!$do_migration)
			return DiscloseSecret_MIGRATION_SKIPPED;
		if ($interactive)
			check_admin_referer(DiscloseSecret_ACTION_MIGRATE_PRIVATECATEGORIES);

		// do migration if requested
		$privatecategories_cats_in = "'". implode("', '", $privatecategories_cats). "'";
		$query = "
			SELECT DISTINCT post.ID as post_id
			FROM {$wpdb->posts} AS post, {$wpdb->post2cat} AS post2cat
			WHERE post.ID = post2cat.post_id
			AND post2cat.category_id NOT IN ($privatecategories_cats_in)
			AND post_status IN ('publish')
		";
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();

		$publish = array();
		foreach ($s as $i)
			$publish[] = $i->post_id;
		$publish_in = "'". implode("', '", $publish). "'";

		$query = "
			SELECT ID as post_id
			FROM {$wpdb->posts}
			WHERE post_status NOT IN ('publish')
		";
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();

		$unchanged = array();
		foreach ($s as $i)
			$unchanged[] = $i->post_id;
		$unchanged_in = "'". implode("', '", $unchanged). "'";

		$query = "
			SELECT ID AS post_id
			FROM {$wpdb->posts}
			WHERE ID NOT IN ($unchanged_in)
			AND ID NOT IN ($publish_in)
		";
		$wpdb->query($query);
		$s = $wpdb->get_results($query);
		if (!is_array($s))
			$s = array();

		foreach ($s as $i)
		{
			$query = "
				INSERT INTO {$wpdb->postmeta}
				(post_id, meta_key, meta_value)
				VALUES ({$i->post_id}, '". DiscloseSecret_META_CAP. "'
					, '". DiscloseSecret_CAPABILITY_PREFIX_LEVEL. "$privatecategories_level')
			";
			$wpdb->query($query);
		}

		$query = "
			UPDATE {$wpdb->posts}
			SET post_status = 'private'
			WHERE post_status IN ('publish')
			AND ID NOT IN ($unchanged_in)
			AND ID NOT IN ($publish_in)
		";
		$wpdb->query($query);

		// cleaning up obsolete stuff
		delete_option(DiscloseSecret_PRIVATECATEGORIES_OPTION_CATS);
		delete_option(DiscloseSecret_PRIVATECATEGORIES_OPTION_LEVEL);

		return DiscloseSecret_MIGRATION_DONE;
	}

	function print_form()
	{
		if ($this->m_status == DiscloseSecret_MIGRATION_SKIPPED)
		{
?>
	<div class="wrap">
		<h2><?php _e('Migration from Private Categories', DiscloseSecret_PLUGIN_ID); ?></h2>
		<p><?php _e('Disclose-Secret has found that your blog used a version of the <a href="http://gregelin.com/archives/?page_id=2">Private Categories</a> plugin. If both plugins are active the functionality of each may behave unexpected! You can easily migrate your private categories to Disclose-Secret by pressing the button below.', DiscloseSecret_PLUGIN_ID); ?></p>
		<p><?php _e('Summary: After migration your blog behaves the same way as with Private Categories but is configured differently. Now the author has to take care to restrict the post when he saves it instead of the post being automatically protected by choosing the right categories. You also should consider to install a Role Manager. See the common plugin repositories for implementations.', DiscloseSecret_PLUGIN_ID); ?></p>
		<p><?php _e('Migration was tested with Private Categories 2 version 0.2.', DiscloseSecret_PLUGIN_ID); ?></p>
		<p><strong><?php _e('Note, that migration can not be undone by this plugin once you have pressed the button below.', DiscloseSecret_PLUGIN_ID); ?></strong></p>
		<form action="" method="post" id="<?php echo DiscloseSecret_CONFIG_FORM_ID_PRIVATECATEGORIES; ?>">
			<p class="submit">
				<input type="submit" name="<?php echo DiscloseSecret_ACTION_MIGRATE_PRIVATECATEGORIES; ?>" value="<?php _e('Migrate Private Categories &raquo;', DiscloseSecret_PLUGIN_ID); ?>" />
				<?php wp_nonce_field(DiscloseSecret_ACTION_MIGRATE_PRIVATECATEGORIES); ?>

			</p>
		</form>
	</div>
<?php
		}
	}
}

?>